import json
import math
import pandas as pd
import pydeck as pdk
import requests
import streamlit as st
from dragonfly_parser import get_json_array

from pollination_streamlit_io import (button, inputs)

def deg2num(lat_deg, lon_deg, zoom):
    lat_rad = math.radians(lat_deg)
    n = 2.0 ** zoom
    xtile = int((lon_deg + 180.0) / 360.0 * n)
    ytile = int((1.0 - math.asinh(math.tan(lat_rad)) / math.pi) / 2.0 * n)
    return (xtile, ytile)

query = st.experimental_get_query_params()
platform = query['__platform__'][0] if '__platform__' in query else 'web'

cities = {
    'New York': [40.7495292, -73.9928448],
    'Boston': [42.361145, -71.057083],
    'Sydney': [-33.865143, 151.209900],
    'Rio De Janeiro': [-22.9094545, -43.1823189],
    'London': [51.5072, -0.1276]
}

option = st.selectbox("Samples",  cities.keys())
st.write(f"Lat: {cities[option][0]} ",
         f"Lon: {cities[option][1]}")

lat = st.number_input(
    "Latitude (deg)",
    min_value=-90.0,
    max_value=90.0,
    value=cities[option][0], step=0.1,
    format="%f")
lon = st.number_input(
    "Longitude (deg)",
    min_value=-180.0,
    max_value=180.0,
    value=cities[option][1],
    step=0.1,
    format="%f")
zoom = st.number_input(
    "Zoom (15 or 16)",
    min_value=15,
    max_value=16,
    value=15,
    step=1)

x, y = deg2num(lat, lon, zoom)
DATA_URL = f"https://data.osmbuildings.org/0.2/anonymous/tile/{zoom}/{x}/{y}.json"
print(DATA_URL)
print(lat, lon)

text_content, lbt_text_content = None, None
out_city, out_lbt_city = "city.json", "lbt_city.json"
try:
    req = requests.get(DATA_URL)
    with open(out_city, "w") as f:
        text_content = json.dumps(req.json())
except requests.exceptions.RequestException as e:
    st.error(f"{e}")
except Exception as e:
    st.error("Geojson not found.")

json_out = None
try:
    json_out = get_json_array(req.json(), lat, lon)
    with open(out_lbt_city, "w") as f:
        lbt_text_content = json_out
except Exception as e:
    st.error("Convert to LBT failed.")

if text_content:
    st.download_button(
        label='Download Geojson',
        data=text_content,
        file_name=out_city)

if lbt_text_content:
    st.download_button(
        label='Download Ladybug Json',
        data=lbt_text_content,
        file_name=out_lbt_city)


if platform == 'Rhino' and json_out:
    button.send('BakeGeometry',
        json.loads(json_out), 'my-secret-key', key='my-secret-key')
    
    inputs.send(json.loads(json_out), 
        'my-super-secret-key', 
        options={"layer":"StreamlitLayer"}, 
        key='0004')

st.pydeck_chart(pdk.Deck(map_style='mapbox://styles/mapbox/light-v9',
    initial_view_state=pdk.ViewState(latitude=lat,
        longitude=lon,
        zoom=zoom,
        max_zoom=18,
        pitch=45,
        bearing=0),
    layers=[pdk.Layer('GeoJsonLayer',
            DATA_URL,
            opacity=0.5,
            stroked=False,
            filled=True,
            extruded=True,
            wireframe=True,
            get_elevation='properties.height',
            get_fill_color='[255, 0, 0]',
            get_line_color=[255, 255, 255],
            pickable=True),]))



import json

try:  # import the ladybug dependencies
    from ladybug.location import Location
    from ladybug_geometry.geometry2d.pointvector import Point2D
    from ladybug_geometry.geometry3d.face import Face3D
    from ladybug_geometry.geometry3d.mesh import Mesh3D
    from ladybug_geometry.geometry3d.pointvector import Point3D
    from ladybug_geometry.geometry3d.polyface import Polyface3D
except ImportError as e:
    raise ImportError('\nFailed to import ladybug_geometry:\n\t{}'.format(e))

try:  # import the core dragonfly dependencies
    from dragonfly.projection import (lon_lat_to_polygon,
                                      meters_to_long_lat_factors,
                                      origin_long_lat_from_location)
except ImportError as e:
    raise ImportError('\nFailed to import dragonfly:\n\t{}'.format(e))

def get_json_array(data, lat, lon):
    # set base point and location
    pt = Point2D(0, 0)
    location = Location(longitude=lon, latitude=lat)

    # parse the geoJSON into a dictionary and get lat/lon converters
    origin_lon_lat = origin_long_lat_from_location(location, pt)
    _convert_facs = meters_to_long_lat_factors(origin_lon_lat)
    convert_facs = 1 / _convert_facs[0], 1 / _convert_facs[1]

    # get all features
    geo_types = ('LineString', 'Polygon')
    geo_data = [geo for geo in data['features'] if 'geometry' in geo
                and geo['geometry']['type'] in geo_types and
                'height' in geo['properties']]

    height = [geo['properties']['height'] for 
              geo in data['features'] if 'height' in geo['properties']]

    # convert all of the geoJSON to ladybug geometry
    buildings = []
    for geo_dat, h in zip(geo_data, height):
        coords = lon_lat_to_polygon(geo_dat['geometry']['coordinates'][0],
                                    origin_lon_lat, convert_facs)
        pts = tuple(Point3D(pt[0], pt[1], 0) for pt in coords)
        face = Face3D(pts, enforce_right_hand=True)
        polyface = Polyface3D.from_offset_face(face, -h)

        buildings.append(polyface.to_dict())

    return json.dumps(buildings)



import streamlit as st
import json

from pollination_streamlit_io import (button, 
    inputs,
    special)

# get the platform from the query uri
query = st.experimental_get_query_params()
platform = query['__platform__'][0] if '__platform__' in query else 'web'

if platform == 'Rhino':
    # special controls
    st.subheader('Pollination Token for Sync')
    po_token = special.sync(key="my-po-sync")
    st.write(po_token)

    st.subheader('Stremlit Token for Sync')
    st_token = st.slider(
        "streamlit token",
        min_value=1,
        max_value=10,
        value=1, 
        step=1)
    
    options = {
        'Pollination Token': po_token,
        'Streamlit Token': "PollinationModel" + str(st_token),
    }
    selection = st.selectbox(label="Select token type: ", options=
    options.keys())

    # common controls
    st.subheader('Pollination, Get Geometry Button')
    geometry = button.get(key='0001')
    if geometry:
        st.write(json.loads(geometry))

    st.subheader('Pollination, Get Pollination Model Button Sync')
    model = button.get(isPollinationModel=True,
        syncToken=options[selection],
        key='0002-1')
    if model:
        st.write(json.loads(model))

    st.subheader('Pollination, Get Pollination Model Button')
    model = button.get(isPollinationModel=True, key='0002-2')
    if model:
        st.write(json.loads(model))

    st.subheader('Pollination, Bake Geometry Button')

    data_to_pass = [{
            'type': 'Mesh3D',
            'vertices': [(0, 0, 0), (10, 0, 0), (0, 10, 0)],
            'faces': [(0, 1, 2)],
            'colors': [{'r': 255, 'g': 0, 'b': 0}]
        }, 
        { 
            'type': 'Polyline2D',
                'vertices': [[0, 0], [10, 0], [0, 10]] 
        }]

    option = st.selectbox(
        'What command do you want to use?',
        ('BakeGeometry', 'ClearGeometry', 'DrawGeometry', 
        'DisableDraw', 'WrongCommand'))
    command_active = button.send(option,
        data_to_pass, 'my-secret-key', 
        options={"layer":"StreamlitLayer", "units": "Feet"}, 
        key='0003')
    st.write('Command in action: %s !' % command_active)
        
    st.write(data_to_pass)

    st.subheader('Pollination, Display Checkbox')

    inputs.send(data_to_pass, 
        'my-secret-key', 
        options={"layer":"StreamlitLayer"}, 
        key='0004')

    data_model = model if model else '{}'

    command_model = button.send('BakePollinationModel',
        json.loads(data_model), 'my-secret-key', key='0005')

    st.subheader('Pollination, Command Button')

    name_input = st.text_input('Enter the command here!', value='PO_AddRooms')
    command = button.command(commandString=name_input, key='0006')
    st.write(command)



import os
import streamlit.components.v1 as components

__all__ = ['send']

_RELEASE = True

if not _RELEASE:
    _component_func_send = components.declare_component(
        "set",
        url="http://localhost:3004",
    )
else:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "send")
    _component_func_send = components.declare_component("send", path=build_dir)

def send(data: dict, 
    uniqueId: str, 
    options: dict = {},
    key=None) -> str:
    """Create a new instance of "button.send".

    Parameters
    ----------
    data: dict
        A Python dictionary. When you run Pollination command it MUST be Pollination model
        Otherwise it MUST be ladybug geometry dictionary (see. to_dict).
    uniqueId: str
        A key to recognize what geometries come from streamlit on Rhino. It becomes 
        a userString inside Rhino.
    options: dict
        A Python dictionary to specify options to use for baking geometry.
        If you use BakePollinationModel you do not need layer options.
        .. code-block:: python
            {
                "layer": "My-Custom-Layer",
                "units': "Meters"
            }
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.
    """
    component_value = _component_func_send(
        data=data,
        uniqueId=uniqueId,
        options=options,
        key=key, 
        default="NAN")

    return component_value

if not _RELEASE:
    import streamlit as st
    import json

    st.subheader("Magic Checkbox")

    data_to_pass = [{
            "type": "Mesh3D",
            "vertices": [(0, 0, 0), (10, 0, 0), (0, 10, 0)],
            "faces": [(0, 1, 2)],
            "colors": [{"r": 255, "g": 0, "b": 0}]
        }, 
        { 
            'type': 'Polyline2D',
             'vertices': [[0, 0], [10, 0], [0, 10]] 
        }]

    send(
        data=data_to_pass, 
        uniqueId="my-secret-key", 
        options={
            "layer": "MyCustomLayer", 
            "units": "Feet"
            },
        key="secret-key-1")




import setuptools

with open('requirements.txt') as f:
    requirements = f.read().splitlines()


setuptools.setup(
    name="pollination-streamlit-io",
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    author="Ladybug Tools",
    author_email="info@ladybug.tools",
    description="Pollination input/output components for Streamlit",
    long_description="Pollination input/output components for Streamlit to use with pollination apps",
    long_description_content_type="text/plain",
    url="https://github.com/pollination/pollination-streamlit-io",
    packages=setuptools.find_packages(exclude=["tests/*"]),
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=requirements,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent"
    ],
    license="Apache-2.0 License"
)



import os
import streamlit.components.v1 as components

__all__ = ['sync']

_RELEASE = True

parent_dir = os.path.dirname(os.path.abspath(__file__))
build_dir = os.path.join(parent_dir, "sync")
_component_func_send = components.declare_component("sync", path=build_dir)

def sync(key=None) -> str:
    """Create a sync token generator.

    Parameters
    ----------
    key: str or None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.
    Returns
    -------
    str
        Sync token to connect to component to enable the syncronization.
    """
    component_value = _component_func_send(
        key=key, 
        default=None)

    return component_value

if not _RELEASE:
    import streamlit as st

    st.subheader("Sync Token in action")

    sync_token = sync(key="secret-key")
    st.write(sync_token)
